@@ -1,5 +1,5 @@
 /*****************************************************************************
- * Copyright (C) 2013 x265 project
+ * Copyright (C) 2013-2017 MulticoreWare, Inc
  *
  * Authors: Steve Borho <steve@borho.org>
  *
@@ -171,7 +171,7 @@ int x265_encoder_reconfig(x265_encoder* enc, x265_param* param_in)
 
     x265_param save;
     Encoder* encoder = static_cast<Encoder*>(enc);
-    if (encoder->m_reconfigure) /* Reconfigure in progress */
+    if (encoder->m_reconfigure || encoder->m_reconfigureRc) /* Reconfigure in progress */
         return 1;
     memcpy(&save, encoder->m_latestParam, sizeof(x265_param));
     int ret = encoder->reconfigureParam(encoder->m_latestParam, param_in);
@@ -197,7 +197,22 @@ int x265_encoder_reconfig(x265_encoder* enc, x265_param* param_in)
                 return -1;
             }
         }
-        encoder->m_reconfigure = true;
+        if (encoder->m_reconfigureRc)
+        {
+            VPS saveVPS;
+            memcpy(&saveVPS.ptl, &encoder->m_vps.ptl, sizeof(saveVPS.ptl));
+            determineLevel(*encoder->m_latestParam, encoder->m_vps);
+            if (saveVPS.ptl.profileIdc != encoder->m_vps.ptl.profileIdc || saveVPS.ptl.levelIdc != encoder->m_vps.ptl.levelIdc
+                || saveVPS.ptl.tierFlag != encoder->m_vps.ptl.tierFlag)
+            {
+                x265_log(encoder->m_param, X265_LOG_WARNING, "Profile/Level/Tier has changed from %d/%d/%s to %d/%d/%s.Cannot reconfigure rate-control.\n",
+                         saveVPS.ptl.profileIdc, saveVPS.ptl.levelIdc, saveVPS.ptl.tierFlag ? "High" : "Main", encoder->m_vps.ptl.profileIdc,
+                         encoder->m_vps.ptl.levelIdc, encoder->m_vps.ptl.tierFlag ? "High" : "Main");
+                encoder->m_reconfigureRc = false;
+            }
+        }
+        else
+            encoder->m_reconfigure = true;
         encoder->printReconfigureParams();
     }
     return ret;
@@ -222,6 +237,7 @@ int x265_encoder_encode(x265_encoder *enc, x265_nal **pp_nal, uint32_t *pi_nal,
     // encoder now owns these analysisData buffers.
     if (pic_in)
     {
+        pic_in->analysisData.wt = NULL;
         pic_in->analysisData.intraData = NULL;
         pic_in->analysisData.interData = NULL;
         pic_in->analysis2Pass.analysisFramedata = NULL;
